@@ -802,7 +802,7 @@ totem_embedded_set_time (TotemEmbedded *emb,
 {
 	g_message ("totem_embedded_set_time: %"G_GUINT64_FORMAT, time);
 
-	bacon_video_widget_seek_time (emb->bvw, time, NULL);
+	bacon_video_widget_seek_time (emb->bvw, time, FALSE, NULL);
 
 	return TRUE;
 }
@@ -913,6 +913,7 @@ totem_embedded_open_playlist_item (TotemEmbedded *emb,
 			g_message ("Seeking to %d seconds for starttime", plitem->starttime);
 			retval = bacon_video_widget_seek_time (emb->bvw,
 							       plitem->starttime * 1000,
+							       FALSE,
 							       NULL /* FIXME */);
 			if (!retval)
 				return TRUE;
@@ -1959,6 +1960,9 @@ totem_embedded_construct (TotemEmbedded *emb,
 	if (!emb->hidden) {
 		gtk_widget_set_size_request (emb->window, width, height);
 		emb->scrsaver = totem_scrsaver_new ();
+		g_object_set (emb->scrsaver,
+			      "reason", _("Playing a movie"),
+			      NULL);
 	}
 
 #ifdef GNOME_ENABLE_DEBUG
