@@ -31,11 +31,14 @@ TODO:
 """
 
 import os
-import totem
-import gconf
 import gobject
-import gtk
-import pango
+from gi.repository import GConf
+from gi.repository import Peas
+from gi.repository import PeasUI
+from gi.repository import Gtk
+from gi.repository import GdkPixbuf
+from gi.repository import Totem
+from gi.repository import Pango
 import socket
 import threading
 import time
@@ -48,9 +51,9 @@ except ImportError:
     try:
         import simplejson as json
     except ImportError:
-        dlg = gtk.MessageDialog(
-            type=gtk.MESSAGE_ERROR,
-            buttons=gtk.BUTTONS_OK
+        dlg = Gtk.MessageDialog(
+            message_type=Gtk.MessageType.ERROR,
+            buttons=Gtk.ButtonsType.OK
         )
         dlg.set_markup(_('You need to install the Python simplejson module.'))
         dlg.run()
@@ -61,8 +64,9 @@ socket.setdefaulttimeout(30)
 gobject.threads_init()
 gconf_key = '/apps/totem/plugins/jamendo'
 
+class JamendoPlugin(gobject.GObject, Peas.Activatable, PeasUI.Configurable):
+    __gtype_name__ = 'JamendoPlugin'
 
-class JamendoPlugin(totem.Plugin):
     """
     Jamendo totem plugin GUI.
     """
@@ -73,77 +77,110 @@ class JamendoPlugin(totem.Plugin):
     TAB_LATEST      = 2
 
     def __init__(self):
-        totem.Plugin.__init__(self)
         self.debug = True
         self.gstreamer_plugins_present = True
         self.totem = None
-        self.gconf = gconf.client_get_default()
+        self.gconf = GConf.Client.get_default()
         self.init_settings()
 
-    def activate(self, totem_object):
+    def do_activate(self, totem_object):
         """
         Plugin activation.
         """
         # Initialise the interface
-        self.builder = self.load_interface("jamendo.ui", True,
-            totem_object.get_main_window(), self)
-        self.config_dialog = self.builder.get_object('config_dialog')
-        self.popup = self.builder.get_object('popup_menu')
-        container = self.builder.get_object('container')
-        self.notebook = self.builder.get_object('notebook')
-        self.search_entry = self.builder.get_object('search_entry')
-        self.search_combo = self.builder.get_object('search_combo')
+        builder = Totem.plugin_load_interface ("jamendo", "jamendo.ui", True, totem_object.get_main_window (), self)
+        self.popup = builder.get_object('popup_menu')
+        container = builder.get_object('container')
+        self.notebook = builder.get_object('notebook')
+        self.search_entry = builder.get_object('search_entry')
+        self.search_combo = builder.get_object('search_combo')
         self.search_combo.set_active(0)
-        self.album_button = self.builder.get_object('album_button')
-        self.previous_button = self.builder.get_object('previous_button')
-        self.next_button = self.builder.get_object('next_button')
+        self.album_button = builder.get_object('album_button')
+        self.previous_button = builder.get_object('previous_button')
+        self.next_button = builder.get_object('next_button')
         self.progressbars = [
-            self.builder.get_object('results_progressbar'),
-            self.builder.get_object('popular_progressbar'),
-            self.builder.get_object('latest_progressbar'),
+            builder.get_object('results_progressbar'),
+            builder.get_object('popular_progressbar'),
+            builder.get_object('latest_progressbar'),
         ]
         self.treeviews = [
-            self.builder.get_object('results_treeview'),
-            self.builder.get_object('popular_treeview'),
-            self.builder.get_object('latest_treeview'),
+            builder.get_object('results_treeview'),
+            builder.get_object('popular_treeview'),
+            builder.get_object('latest_treeview'),
         ]
         self.setup_treeviews()
 
         # Set up signals
-        self.builder.get_object('search_button').connect('clicked', self.on_search_button_clicked)
+        builder.get_object('search_button').connect('clicked', self.on_search_button_clicked)
         self.search_entry.connect('activate', self.on_search_entry_activate)
         self.notebook.connect('switch-page', self.on_notebook_switch_page)
         self.previous_button.connect('clicked', self.on_previous_button_clicked)
         self.next_button.connect('clicked', self.on_next_button_clicked)
         self.album_button.connect('clicked', self.on_album_button_clicked)
-        self.builder.get_object('cancel_button').connect('clicked', self.on_cancel_button_clicked)
-        self.builder.get_object('ok_button').connect('clicked', self.on_ok_button_clicked)
-        self.builder.get_object('add_to_playlist').connect('activate', self.on_add_to_playlist_activate)
-        self.builder.get_object('jamendo_album_page').connect('activate', self.on_open_jamendo_album_page_activate)
+        builder.get_object('add_to_playlist').connect('activate', self.on_add_to_playlist_activate)
+        builder.get_object('jamendo_album_page').connect('activate', self.on_open_jamendo_album_page_activate)
 
         self.totem = totem_object
         self.reset()
         container.show_all()
         self.totem.add_sidebar_page("jamendo", _("Jamendo"), container)
 
-    def deactivate(self, totem_object):
+    def do_deactivate(self, totem_object):
         """
         Plugin deactivation.
         """
         totem_object.remove_sidebar_page("jamendo")
 
-    def create_configure_dialog(self, *args):
+    def do_create_configure_widget(self):
         """
-        Plugin config dialog.
+        Plugin config widget.
+        This code must be independent from the rest of the plugin. FIXME: bgo#624073
         """
-        format = self.gconf.get_string('%s/format' % gconf_key)
-        num_per_page = self.gconf.get_int('%s/num_per_page' % gconf_key)
-        combo = self.builder.get_object('preferred_format_combo')
+        builder = Totem.plugin_load_interface ('jamendo', 'jamendo.ui', True, None, self)
+        print builder
+        gconf = GConf.Client.get_default()
+        config_widget = builder.get_object ('config_widget')
+        config_widget.connect ('destroy', self.on_config_widget_destroy)
+        print config_widget
+        format = gconf.get_string('%s/format' % gconf_key)
+        num_per_page = gconf.get_int('%s/num_per_page' % gconf_key)
+
+        combo = builder.get_object('preferred_format_combo')
         combo.set_active(self.AUDIO_FORMATS.index(format))
-        spinbutton = self.builder.get_object('album_num_spinbutton')
+        combo.connect ('changed', self.on_preferred_format_combo_changed)
+
+        spinbutton = builder.get_object('album_num_spinbutton')
         spinbutton.set_value(num_per_page)
-        self.config_dialog.set_default_response(gtk.RESPONSE_OK)
-        return self.config_dialog
+        spinbutton.connect ('value-changed', self.on_album_num_spinbutton_value_changed)
+
+        return config_widget
+
+    def on_preferred_format_combo_changed (self, combo):
+        """
+        Called when the preferred audio format combo box is changed in the configuration dialogue
+        """
+        gconf = GConf.Client.get_default()
+        format = self.AUDIO_FORMATS[combo.get_active()]
+        gconf.set_string('%s/format' % gconf_key, format)
+
+    def on_album_num_spinbutton_value_changed (self, spinbutton):
+        """
+        Called when the number-of-albums spinbutton is changed in the configuration dialogue
+        """
+        gconf = GConf.Client.get_default()
+        num_per_page = int(spinbutton.get_value())
+        gconf.set_int('%s/num_per_page' % gconf_key, num_per_page)
+
+    def on_config_widget_destroy (self, widget):
+        """
+        FIXME: The GConf stuff should be refactored so that it listens to notifications and doesn't update manually.
+        This function is hacky and should go away.
+        """
+        self.init_settings()
+        try:
+            self.reset()
+        except:
+            pass
 
     def reset(self):
         """
@@ -190,33 +227,28 @@ class JamendoPlugin(totem.Plugin):
         """
         self.current_treeview = self.treeviews[0]
         for w in self.treeviews:
-            w.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
+            w.get_selection().set_mode(Gtk.SelectionMode.MULTIPLE)
 
             # build pixbuf column
-            cell = gtk.CellRendererPixbuf()
-            col = gtk.TreeViewColumn()
-            col.pack_start(cell, True)
-            col.set_attributes(cell, pixbuf=1)
+            cell = Gtk.CellRendererPixbuf()
+            col = Gtk.TreeViewColumn(cell_renderer=cell, pixbuf=1)
+
             w.append_column(col)
 
             # build description column
-            cell = gtk.CellRendererText()
-            cell.set_property('wrap-mode', pango.WRAP_WORD)
+            cell = Gtk.CellRendererText()
+            cell.set_property('wrap-mode', Pango.WrapMode.WORD)
             cell.set_property('wrap-width', 30)
-            col = gtk.TreeViewColumn()
-            col.pack_start(cell, True)
-            col.set_attributes(cell, markup=2)
+            col = Gtk.TreeViewColumn(cell_renderer=cell, markup=2)
             col.set_expand(True)
             w.append_column(col)
             w.connect_after('size-allocate', self.on_treeview_size_allocate, col, cell)
 
             # duration column
-            cell = gtk.CellRendererText()
+            cell = Gtk.CellRendererText()
             cell.set_property('xalign', 1.0)
             cell.set_property('size-points', 8)
-            col = gtk.TreeViewColumn()
-            col.pack_start(cell, True)
-            col.set_attributes(cell, markup=3)
+            col = Gtk.TreeViewColumn(cell_renderer=cell, markup=3)
             col.set_alignment(1.0)
             w.append_column(col)
 
@@ -230,15 +262,15 @@ class JamendoPlugin(totem.Plugin):
 
 
     def add_treeview_item(self, treeview, album):
-        if not isinstance(album['image'], gtk.gdk.Pixbuf):
+        if not isinstance(album['image'], GdkPixbuf.Pixbuf):
             # album image pixbuf is not yet built
             try:
-                pb = gtk.gdk.pixbuf_new_from_file(album['image'])
+                pb = GdkPixbuf.Pixbuf.new_from_file(album['image'])
                 os.unlink(album['image'])
                 album['image'] = pb
             except:
                 # do not fail for this, just display a dummy pixbuf
-                album['image'] = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, True,
+                album['image'] = GdkPixbuf.Pixbuf.new(GdkPixbuf.Colorspace.RGB, True,
                     8, 1, 1)
         # format title
         title  = '<b>%s</b>\n' % self._format_str(album['name'])
@@ -265,7 +297,7 @@ class JamendoPlugin(totem.Plugin):
             [album, album['image'], title, dur, tip]
         )
         # append track rows
-        icon = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, True, 8, 1, 1)
+        icon = GdkPixbuf.Pixbuf.new(GdkPixbuf.Colorspace.RGB, True, 8, 1, 1)
         for i, track in enumerate(album['tracks']):
             # track title
             # Translators: this is the title of a track in Python format
@@ -309,9 +341,9 @@ class JamendoPlugin(totem.Plugin):
         enqueue_and_play.
         """
         if mode == 'replace':
-            self.totem.action_remote(totem.REMOTE_COMMAND_REPLACE, t['stream'])
+            self.totem.action_remote(Totem.RemoteCommand.REPLACE, t['stream'])
         elif mode == 'enqueue':
-            self.totem.action_remote(totem.REMOTE_COMMAND_ENQUEUE, t['stream'])
+            self.totem.action_remote(Totem.RemoteCommand.ENQUEUE, t['stream'])
 
     def fetch_albums(self, pn=1):
         """
@@ -380,13 +412,7 @@ class JamendoPlugin(totem.Plugin):
         self.progressbars[pindex].set_fraction(0.0)
         self.progressbars[pindex].hide()
         self.running_threads[pindex] = False
-        dlg = gtk.MessageDialog(
-            type=gtk.MESSAGE_ERROR,
-            buttons=gtk.BUTTONS_OK
-        )
-        dlg.set_markup(
-            '<b>%s</b>' % _('An error occurred while fetching albums.')
-        )
+
         # managing exceptions with urllib is a real PITA... :(
         if hasattr(exc, 'reason'):
             try:
@@ -402,9 +428,8 @@ class JamendoPlugin(totem.Plugin):
             msg = _('The Jamendo server returned code %s.') % exc.code
         else:
             msg = str(exc)
-        dlg.format_secondary_text(msg)
-        dlg.run()
-        dlg.destroy()
+
+        self.totem.action_error(_('An error occurred while fetching albums.'), msg)
 
     def on_search_entry_activate(self, *args):
         """
@@ -465,7 +490,7 @@ class JamendoPlugin(totem.Plugin):
             if evt.button == 3:
                 path = tv.get_path_at_pos(int(evt.x), int(evt.y))
                 sel  = tv.get_selection()
-                rows = sel.get_selected_rows()
+                (rows, _) = sel.get_selected_rows()
                 if path[0] not in rows[1]:
                     sel.unselect_all()
                     sel.select_path(path[0])
@@ -534,29 +559,6 @@ class JamendoPlugin(totem.Plugin):
         except:
             pass
 
-    def on_cancel_button_clicked(self, *args):
-        """
-        Called when the user clicked cancel in the config dialog.
-        """
-        self.config_dialog.hide()
-
-    def on_ok_button_clicked(self, *args):
-        """
-        Called when the user clicked ok in the config dialog.
-        """
-        combo = self.builder.get_object('preferred_format_combo')
-        spinbutton = self.builder.get_object('album_num_spinbutton')
-        format = self.AUDIO_FORMATS[combo.get_active()]
-        self.gconf.set_string('%s/format' % gconf_key, format)
-        num_per_page = int(spinbutton.get_value())
-        self.gconf.set_int('%s/num_per_page' % gconf_key, num_per_page)
-        self.init_settings()
-        self.config_dialog.hide()
-        try:
-            self.reset()
-        except:
-            pass
-
     def on_add_to_playlist_activate(self, *args):
         """
         Called when the user clicked on the add to playlist button of the
@@ -584,7 +586,7 @@ class JamendoPlugin(totem.Plugin):
         """
         ret = []
         sel = self.current_treeview.get_selection()
-        model, rows = sel.get_selected_rows()
+        (rows, model) = sel.get_selected_rows()
         for row in rows:
             if root:
                 it = model.get_iter((row[0],))
@@ -600,7 +602,7 @@ class JamendoPlugin(totem.Plugin):
         Update the state of the previous and next buttons.
         """
         sel = self.current_treeview.get_selection()
-        model, rows = sel.get_selected_rows()
+        (rows, model) = sel.get_selected_rows()
         try:
             it = model.get_iter(rows[0])
         except:
