@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/* 
- * Copyright (C) 2009 Philip Withnall <philip@tecnocode.co.uk>
+/*
+ * Copyright (C) 2009–2010 Philip Withnall <philip@tecnocode.co.uk>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,10 +28,14 @@
 #include <glib.h>
 #include <glib-object.h>
 #include <glib/gi18n-lib.h>
+#include <libpeas/peas-extension-base.h>
+#include <libpeas/peas-object-module.h>
+#include <libpeas/peas-activatable.h>
 #include <gdata/gdata.h>
 
 #include "totem-plugin.h"
 #include "totem.h"
+#include "totem-dirs.h"
 #include "totem-video-list.h"
 #include "totem-interface.h"
 #include "backend/bacon-video-widget.h"
@@ -58,7 +62,7 @@ enum {
 #define TOTEM_YOUTUBE_PLUGIN_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), TOTEM_TYPE_YOUTUBE_PLUGIN, TotemYouTubePluginClass))
 
 typedef struct {
-	TotemPlugin parent;
+	PeasExtensionBase parent;
 	Totem *totem;
 	GDataYouTubeService *service;
 	BaconVideoWidget *bvw;
@@ -83,14 +87,10 @@ typedef struct {
 } TotemYouTubePlugin;
 
 typedef struct {
-	TotemPluginClass parent_class;
+	PeasExtensionBaseClass parent_class;
 } TotemYouTubePluginClass;
 
-G_MODULE_EXPORT GType register_totem_plugin	(GTypeModule *module);
-GType totem_youtube_plugin_get_type		(void) G_GNUC_CONST;
-
-static gboolean impl_activate			(TotemPlugin *plugin, TotemObject *totem, GError **error);
-static void impl_deactivate			(TotemPlugin *plugin, TotemObject *totem);
+GType totem_youtube_plugin_get_type (void) G_GNUC_CONST;
 
 /* GtkBuilder callbacks */
 void notebook_switch_page_cb (GtkNotebook *notebook, GtkNotebookPage *page, guint page_num, TotemYouTubePlugin *self);
@@ -103,27 +103,22 @@ void open_in_web_browser_activate_cb (GtkAction *action, TotemYouTubePlugin *sel
 void value_changed_cb (GtkAdjustment *adjustment, TotemYouTubePlugin *self);
 gboolean starting_video_cb (TotemVideoList *video_list, GtkTreePath *path, TotemYouTubePlugin *self);
 
-TOTEM_PLUGIN_REGISTER (TotemYouTubePlugin, totem_youtube_plugin)
+TOTEM_PLUGIN_REGISTER (TOTEM_TYPE_YOUTUBE_PLUGIN, TotemYouTubePlugin, totem_youtube_plugin);
 
 static void
 totem_youtube_plugin_class_init (TotemYouTubePluginClass *klass)
 {
-	TotemPluginClass *plugin_class = TOTEM_PLUGIN_CLASS (klass);
-
-	plugin_class->activate = impl_activate;
-	plugin_class->deactivate = impl_deactivate;
 }
 
 static void
 totem_youtube_plugin_init (TotemYouTubePlugin *plugin)
 {
-	/* Nothing to see here; move along */
 }
 
 /* ----------------------------------------------------------------------------------------------------------------- */
 /* Copied from http://bugzilla.gnome.org/show_bug.cgi?id=575900 while waiting for them to be committed to gdk-pixbuf */
 
-typedef	struct {
+typedef struct {
 	gint width;
 	gint height;
 	gboolean preserve_aspect_ratio;
@@ -131,8 +126,8 @@ typedef	struct {
 
 static void
 new_from_stream_thread (GSimpleAsyncResult *result,
-			GInputStream       *stream,
-			GCancellable       *cancellable)
+                        GInputStream       *stream,
+                        GCancellable       *cancellable)
 {
 	GdkPixbuf *pixbuf;
 	AtScaleData *data;
@@ -164,9 +159,9 @@ new_from_stream_thread (GSimpleAsyncResult *result,
  * @preserve_aspect_ratio: %TRUE to preserve the image's aspect ratio
  * @cancellable: optional #GCancellable object, %NULL to ignore
  * @callback: a #GAsyncReadyCallback to call when the the pixbuf is loaded
- * @user_data: the data to pass to the callback function 
+ * @user_data: the data to pass to the callback function
  *
- * Creates a new pixbuf by asynchronously loading an image from an input stream.  
+ * Creates a new pixbuf by asynchronously loading an image from an input stream.
  *
  * For more details see gdk_pixbuf_new_from_stream_at_scale(), which is the synchronous
  * version of this function.
@@ -178,12 +173,12 @@ new_from_stream_thread (GSimpleAsyncResult *result,
  **/
 static void
 totem_gdk_pixbuf_new_from_stream_at_scale_async (GInputStream        *stream,
-					   gint                 width,
-					   gint                 height,
-					   gboolean             preserve_aspect_ratio,
-					   GCancellable        *cancellable,
-					   GAsyncReadyCallback  callback,
-					   gpointer             user_data)
+                                                 gint                 width,
+                                                 gint                 height,
+                                                 gboolean             preserve_aspect_ratio,
+                                                 GCancellable        *cancellable,
+                                                 GAsyncReadyCallback  callback,
+                                                 gpointer             user_data)
 {
 	GSimpleAsyncResult *result;
 	AtScaleData *data;
@@ -207,9 +202,9 @@ totem_gdk_pixbuf_new_from_stream_at_scale_async (GInputStream        *stream,
  * @stream: a #GInputStream from which to load the pixbuf
  * @cancellable: optional #GCancellable object, %NULL to ignore
  * @callback: a #GAsyncReadyCallback to call when the the pixbuf is loaded
- * @user_data: the data to pass to the callback function 
+ * @user_data: the data to pass to the callback function
  *
- * Creates a new pixbuf by asynchronously loading an image from an input stream.  
+ * Creates a new pixbuf by asynchronously loading an image from an input stream.
  *
  * For more details see gdk_pixbuf_new_from_stream(), which is the synchronous
  * version of this function.
@@ -221,9 +216,9 @@ totem_gdk_pixbuf_new_from_stream_at_scale_async (GInputStream        *stream,
  **/
 static void
 totem_gdk_pixbuf_new_from_stream_async (GInputStream        *stream,
-				  GCancellable        *cancellable,
-				  GAsyncReadyCallback  callback,
-				  gpointer             user_data)
+                                        GCancellable        *cancellable,
+                                        GAsyncReadyCallback  callback,
+                                        gpointer             user_data)
 {
 	GSimpleAsyncResult *result;
 
@@ -250,14 +245,14 @@ totem_gdk_pixbuf_new_from_stream_async (GInputStream        *stream,
  **/
 static GdkPixbuf *
 totem_gdk_pixbuf_new_from_stream_finish (GAsyncResult  *async_result,
-				   GError       **error)
+                                         GError       **error)
 {
 	GdkPixbuf *pixbuf;
 	GSimpleAsyncResult *result = G_SIMPLE_ASYNC_RESULT (async_result);
 
 	g_return_val_if_fail (G_IS_ASYNC_RESULT (async_result), NULL);
 	g_warn_if_fail (g_simple_async_result_get_source_tag (result) == totem_gdk_pixbuf_new_from_stream_async ||
-			g_simple_async_result_get_source_tag (result) == totem_gdk_pixbuf_new_from_stream_at_scale_async);
+	                g_simple_async_result_get_source_tag (result) == totem_gdk_pixbuf_new_from_stream_at_scale_async);
 
 	if (g_simple_async_result_propagate_error (result, error))
 		return NULL;
@@ -284,8 +279,8 @@ set_up_tree_view (TotemYouTubePlugin *self, GtkBuilder *builder, guint key)
 	/* Add the cell renderer. This can't be done with GtkBuilder, because it unavoidably sets the expand parameter to FALSE */
 	/* TODO: Depends on bug #453692 */
 	renderer = GTK_CELL_RENDERER (totem_cell_renderer_video_new (TRUE));
-	column = GTK_TREE_VIEW_COLUMN (gtk_builder_get_object (builder,
-							       (key == SEARCH_TREE_VIEW) ? "yt_treeview_search_column" : "yt_treeview_related_column"));
+	column = GTK_TREE_VIEW_COLUMN (gtk_builder_get_object (builder, (key == SEARCH_TREE_VIEW) ? "yt_treeview_search_column"
+	                                                                                          : "yt_treeview_related_column"));
 	gtk_tree_view_column_pack_start (column, renderer, TRUE);
 	gtk_tree_view_column_set_attributes (column, renderer, "thumbnail", 0, "title", 1, NULL);
 
@@ -316,11 +311,11 @@ set_up_tree_view (TotemYouTubePlugin *self, GtkBuilder *builder, guint key)
 
 	gtk_ui_manager_insert_action_group (ui_manager, action_group, 1);
 	gtk_ui_manager_add_ui (ui_manager, gtk_ui_manager_new_merge_id (ui_manager),
-			       "/ui/totem-video-list-popup/",
-			       "open-in-web-browser",
-			       "open-in-web-browser",
-			       GTK_UI_MANAGER_MENUITEM,
-			       FALSE);
+	                       "/ui/totem-video-list-popup/",
+	                       "open-in-web-browser",
+	                       "open-in-web-browser",
+	                       GTK_UI_MANAGER_MENUITEM,
+	                       FALSE);
 
 	menu_item = gtk_ui_manager_get_action (ui_manager, "/ui/totem-video-list-popup/open-in-web-browser");
 	g_signal_connect (menu_item, "activate", G_CALLBACK (open_in_web_browser_activate_cb), self);
@@ -332,10 +327,11 @@ set_up_tree_view (TotemYouTubePlugin *self, GtkBuilder *builder, guint key)
 	self->cancel_button = GTK_WIDGET (gtk_builder_get_object (builder, "yt_cancel_button"));
 }
 
-static gboolean
-impl_activate (TotemPlugin *plugin, TotemObject *totem, GError **error)
+static void
+impl_activate (PeasActivatable *plugin, GObject *object)
 {
 	TotemYouTubePlugin *self = TOTEM_YOUTUBE_PLUGIN (plugin);
+	TotemObject *totem = TOTEM_OBJECT (object);
 	GtkWindow *main_window;
 	GtkBuilder *builder;
 	guint i;
@@ -345,7 +341,7 @@ impl_activate (TotemPlugin *plugin, TotemObject *totem, GError **error)
 
 	/* Set up the interface */
 	main_window = totem_get_main_window (totem);
-	builder = totem_plugin_load_interface (plugin, "youtube.ui", TRUE, main_window, self);
+	builder = totem_plugin_load_interface ("youtube", "youtube.ui", TRUE, main_window, self);
 	g_object_unref (main_window);
 
 	self->search_entry = GTK_ENTRY (gtk_builder_get_object (builder, "yt_search_entry"));
@@ -363,12 +359,10 @@ impl_activate (TotemPlugin *plugin, TotemObject *totem, GError **error)
 	/* Add the sidebar page */
 	totem_add_sidebar_page (totem, "youtube", _("YouTube"), self->vbox);
 	g_object_unref (builder);
-
-	return TRUE;
 }
 
 static void
-impl_deactivate	(TotemPlugin *plugin, TotemObject *totem)
+impl_deactivate (PeasActivatable *plugin, GObject *totem)
 {
 	guint i;
 	TotemYouTubePlugin *self = TOTEM_YOUTUBE_PLUGIN (plugin);
@@ -376,10 +370,10 @@ impl_deactivate	(TotemPlugin *plugin, TotemObject *totem)
 	totem_remove_sidebar_page (self->totem, "youtube");
 
 	for (i = 0; i < NUM_TREE_VIEWS; i++) {
-		if (self->cancellable[i] != NULL) {
+		/* Cancel any queries which are still underway */
+		if (self->cancellable[i] != NULL)
 			g_cancellable_cancel (self->cancellable[i]);
-			g_object_unref (self->cancellable[i]);
-		}
+
 		if (self->query[i] != NULL)
 			g_object_unref (self->query[i]);
 	}
@@ -461,22 +455,13 @@ increment_progress_bar_fraction (TotemYouTubePlugin *self, guint tree_view)
 		gdk_window_set_cursor (gtk_widget_get_window (self->vbox), NULL);
 		gtk_progress_bar_set_text (self->progress_bar[tree_view], "");
 		gtk_progress_bar_set_fraction (self->progress_bar[tree_view], 0.0);
-
-		/* Disable the "Cancel" button, if it applies to the current tree view */
-		if (self->current_tree_view == tree_view)
-			gtk_widget_set_sensitive (self->cancel_button, FALSE);
-
-		/* Unref cancellable */
-		if (self->cancellable[tree_view] != NULL)
-			g_object_unref (self->cancellable[tree_view]);
-		self->cancellable[tree_view] = NULL;
 	}
 }
 
 typedef struct {
 	TotemYouTubePlugin *plugin;
 	GDataEntry *entry;
-	GtkTreeIter iter;
+	GtkTreePath *path;
 	guint tree_view;
 } TParamData;
 
@@ -488,6 +473,7 @@ resolve_t_param_cb (GObject *source_object, GAsyncResult *result, TParamData *da
 	gsize length;
 	GMatchInfo *match_info;
 	GError *error = NULL;
+	GtkTreeIter iter;
 	TotemYouTubePlugin *self = data->plugin;
 
 	/* Finish loading the page */
@@ -538,7 +524,7 @@ resolve_t_param_cb (GObject *source_object, GAsyncResult *result, TParamData *da
 
 		/* We don't have a match, which is odd; fall back to the FLV URI as advertised by the YouTube API */
 		content = GDATA_MEDIA_CONTENT (gdata_youtube_video_look_up_content (GDATA_YOUTUBE_VIDEO (data->entry),
-										    "application/x-shockwave-flash"));
+		                                                                    "application/x-shockwave-flash"));
 		if (content != NULL) {
 			video_uri = g_strdup (gdata_media_content_get_uri (content));
 			g_debug ("Couldn't find the t param of entry %s; falling back to its FLV URI (\"%s\")", video_id, video_uri);
@@ -552,8 +538,10 @@ resolve_t_param_cb (GObject *source_object, GAsyncResult *result, TParamData *da
 	g_free (contents);
 
 	/* Update the tree view with the new MRL */
-	gtk_list_store_set (self->list_store[data->tree_view], &(data->iter), 2, video_uri, -1);
-	g_debug ("Updated list store with new video URI (\"%s\") for entry %s", video_uri, video_id);
+	if (gtk_tree_model_get_iter (GTK_TREE_MODEL (self->list_store[data->tree_view]), &iter, data->path) == TRUE) {
+		gtk_list_store_set (self->list_store[data->tree_view], &iter, 2, video_uri, -1);
+		g_debug ("Updated list store with new video URI (\"%s\") for entry %s", video_uri, video_id);
+	}
 
 	g_free (video_uri);
 
@@ -563,35 +551,37 @@ free_data:
 
 	g_object_unref (data->plugin);
 	g_object_unref (data->entry);
+	gtk_tree_path_free (data->path);
 	g_slice_free (TParamData, data);
 }
 
 static void
-resolve_t_param (TotemYouTubePlugin *self, GDataEntry *entry, GtkTreeIter *iter, guint tree_view)
+resolve_t_param (TotemYouTubePlugin *self, GDataEntry *entry, GtkTreeIter *iter, guint tree_view, GCancellable *cancellable)
 {
-	GDataLink *link;
+	GDataLink *page_link;
 	GFile *video_page;
 	TParamData *data;
 
 	/* We have to get the t parameter from the actual HTML video page, since Google changed how their URIs work */
-	link = gdata_entry_look_up_link (entry, GDATA_LINK_ALTERNATE);
-	g_assert (link != NULL);
+	page_link = gdata_entry_look_up_link (entry, GDATA_LINK_ALTERNATE);
+	g_assert (page_link != NULL);
 
 	data = g_slice_new (TParamData);
 	data->plugin = g_object_ref (self);
 	data->entry = g_object_ref (entry);
-	data->iter = *iter;
+	data->path = gtk_tree_model_get_path (GTK_TREE_MODEL (self->list_store[tree_view]), iter);
 	data->tree_view = tree_view;
 
-	video_page = g_file_new_for_uri (gdata_link_get_uri (link));
-	g_file_load_contents_async (video_page, self->cancellable[tree_view], (GAsyncReadyCallback) resolve_t_param_cb, data);
+	video_page = g_file_new_for_uri (gdata_link_get_uri (page_link));
+	g_file_load_contents_async (video_page, cancellable, (GAsyncReadyCallback) resolve_t_param_cb, data);
 	g_object_unref (video_page);
 }
 
 typedef struct {
 	TotemYouTubePlugin *plugin;
-	GtkTreeIter iter;
+	GtkTreePath *path;
 	guint tree_view;
+	GCancellable *cancellable;
 } ThumbnailData;
 
 static void
@@ -599,6 +589,7 @@ thumbnail_loaded_cb (GObject *source_object, GAsyncResult *result, ThumbnailData
 {
 	GdkPixbuf *thumbnail;
 	GError *error = NULL;
+	GtkTreeIter iter;
 	TotemYouTubePlugin *self = data->plugin;
 
 	/* Finish loading the thumbnail */
@@ -620,8 +611,10 @@ thumbnail_loaded_cb (GObject *source_object, GAsyncResult *result, ThumbnailData
 	g_debug ("Finished creating thumbnail from stream");
 
 	/* Update the tree view */
-	gtk_list_store_set (self->list_store[data->tree_view], &(data->iter), 0, thumbnail, -1);
-	g_debug ("Updated list store with new thumbnail");
+	if (gtk_tree_model_get_iter (GTK_TREE_MODEL (self->list_store[data->tree_view]), &iter, data->path) == TRUE) {
+		gtk_list_store_set (self->list_store[data->tree_view], &iter, 0, thumbnail, -1);
+		g_debug ("Updated list store with new thumbnail");
+	}
 
 	g_object_unref (thumbnail);
 
@@ -630,6 +623,8 @@ free_data:
 	increment_progress_bar_fraction (self, data->tree_view);
 
 	g_object_unref (data->plugin);
+	g_object_unref (data->cancellable);
+	gtk_tree_path_free (data->path);
 	g_slice_free (ThumbnailData, data);
 }
 
@@ -639,7 +634,6 @@ thumbnail_opened_cb (GObject *source_object, GAsyncResult *result, ThumbnailData
 	GFile *thumbnail_file;
 	GFileInputStream *input_stream;
 	GError *error = NULL;
-	TotemYouTubePlugin *self = data->plugin;
 
 	/* Finish opening the thumbnail */
 	thumbnail_file = G_FILE (source_object);
@@ -652,20 +646,40 @@ thumbnail_opened_cb (GObject *source_object, GAsyncResult *result, ThumbnailData
 		return;
 	}
 
+	/* NOTE: There's no need to reset the cancellable before using it again, as we'll have bailed before now if it was ever cancelled. */
+
 	g_debug ("Creating thumbnail from stream");
 	totem_gdk_pixbuf_new_from_stream_at_scale_async (G_INPUT_STREAM (input_stream), THUMBNAIL_WIDTH, -1, TRUE,
-							 self->cancellable[data->tree_view], (GAsyncReadyCallback) thumbnail_loaded_cb, data);
+	                                                 data->cancellable, (GAsyncReadyCallback) thumbnail_loaded_cb, data);
 	g_object_unref (input_stream);
 }
 
 typedef struct {
 	TotemYouTubePlugin *plugin;
 	guint tree_view;
+	GCancellable *query_cancellable;
+	GCancellable *t_param_cancellable;
+	GCancellable *thumbnail_cancellable;
 } QueryData;
 
+static void
+query_data_free (QueryData *data)
+{
+	if (data->thumbnail_cancellable != NULL)
+		g_object_unref (data->thumbnail_cancellable);
+	if (data->t_param_cancellable != NULL)
+		g_object_unref (data->t_param_cancellable);
+
+	g_object_unref (data->query_cancellable);
+	g_object_unref (data->plugin);
+
+	g_slice_free (QueryData, data);
+}
+
 static void
 query_finished_cb (GObject *source_object, GAsyncResult *result, QueryData *data)
 {
+	GtkWindow *window;
 	GDataFeed *feed;
 	GError *error = NULL;
 	TotemYouTubePlugin *self = data->plugin;
@@ -678,38 +692,45 @@ query_finished_cb (GObject *source_object, GAsyncResult *result, QueryData *data
 	self->progress_bar_increment[data->tree_view] = 1.0;
 	increment_progress_bar_fraction (self, data->tree_view);
 
-	if (feed == NULL) {
-		GtkWindow *window;
+	if (feed != NULL) {
+		/* Success! */
+		g_object_unref (feed);
+		query_data_free (data);
 
-		/* Bail out if the operation was cancelled */
-		if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED) == TRUE) {
-			g_error_free (error);
-			goto free_data;
-		}
-
-		/* Error! */
-		window = totem_get_main_window (data->plugin->totem);
-		if (g_error_matches (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_PROTOCOL_ERROR) == TRUE) {
-			/* Hide the ugly technical message libgdata gives behind a nice one telling them it's out of date (which it likely is
-			 * if we're receiving a protocol error). */
-			totem_interface_error (_("Error Searching for Videos"),
-					       _("The response from the server could not be understood. "
-					         "Please check you are running the latest version of libgdata."), window);
-		} else {
-			/* Spew out the error message as provided */
-			totem_interface_error (_("Error Searching for Videos"), error->message, window);
-		}
-
-		g_object_unref (window);
-		g_error_free (error);
-		goto free_data;
+		return;
 	}
 
-	g_object_unref (feed);
+	/* Bail out if the operation was cancelled */
+	if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED) == TRUE) {
+		/* Cancel the t-param and thumbnail threads, if applicable */
+		if (data->t_param_cancellable != NULL)
+			g_cancellable_cancel (data->t_param_cancellable);
+		if (data->thumbnail_cancellable != NULL)
+			g_cancellable_cancel (data->thumbnail_cancellable);
 
-free_data:
-	g_object_unref (data->plugin);
-	g_slice_free (QueryData, data);
+		goto finish;
+	}
+
+	/* Error! */
+	window = totem_get_main_window (data->plugin->totem);
+	if (g_error_matches (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_PROTOCOL_ERROR) == TRUE) {
+		/* Hide the ugly technical message libgdata gives behind a nice one telling them it's out of date (which it likely is
+		 * if we're receiving a protocol error). */
+		totem_interface_error (_("Error Searching for Videos"),
+		                       _("The response from the server could not be understood. "
+		                         "Please check you are running the latest version of libgdata."), window);
+	} else {
+		/* Spew out the error message as provided */
+		totem_interface_error (_("Error Searching for Videos"), error->message, window);
+	}
+
+	g_object_unref (window);
+
+finish:
+	g_error_free (error);
+	query_data_free (data);
+
+	return;
 }
 
 static void
@@ -723,20 +744,17 @@ query_progress_cb (GDataEntry *entry, guint entry_key, guint entry_count, QueryD
 	GtkProgressBar *progress_bar;
 	TotemYouTubePlugin *self = data->plugin;
 
-	/* Check this query hasn't finished */
-	g_assert (self->cancellable[data->tree_view] != NULL);
-
 	/* Add the entry to the tree view */
 	title = gdata_entry_get_title (entry);
 	id = gdata_youtube_video_get_video_id (GDATA_YOUTUBE_VIDEO (entry));
 
 	gtk_list_store_append (self->list_store[data->tree_view], &iter);
 	gtk_list_store_set (self->list_store[data->tree_view], &iter,
-			    0, NULL, /* the thumbnail will be downloaded asynchronously and added to the tree view later */
-			    1, title,
-			    2, NULL, /* the video URI will be resolved asynchronously and added to the tree view later */
-			    3, entry,
-			    -1);
+	                    0, NULL, /* the thumbnail will be downloaded asynchronously and added to the tree view later */
+	                    1, title,
+	                    2, NULL, /* the video URI will be resolved asynchronously and added to the tree view later */
+	                    3, entry,
+	                    -1);
 	g_debug ("Added entry %s to tree view (title: \"%s\")", id, title);
 
 	/* Update the progress bar; we have three steps for each entry in the results: the entry, its thumbnail, and its t parameter */
@@ -747,7 +765,9 @@ query_progress_cb (GDataEntry *entry, guint entry_key, guint entry_count, QueryD
 	gtk_progress_bar_set_fraction (progress_bar, gtk_progress_bar_get_fraction (progress_bar) + self->progress_bar_increment[data->tree_view]);
 
 	/* Resolve the t parameter for the video, which is required before it can be played */
-	resolve_t_param (self, entry, &iter, data->tree_view);
+	/* This will be cancelled if the main query is cancelled, in query_finished_cb() */
+	data->t_param_cancellable = g_cancellable_new ();
+	resolve_t_param (self, entry, &iter, data->tree_view, data->t_param_cancellable);
 
 	/* Download the entry's thumbnail, ready for adding it to the tree view.
 	 * Find the thumbnail size which is closest to the wanted size (THUMBNAIL_WIDTH), so that we:
@@ -764,12 +784,12 @@ query_progress_cb (GDataEntry *entry, guint entry_key, guint entry_count, QueryD
 		if (delta == 0) {
 			break;
 		} else if ((delta == G_MININT) ||
-			   (delta < 0 && new_delta > delta) ||
-			   (delta > 0 && new_delta > 0 && new_delta < delta)) {
+		           (delta < 0 && new_delta > delta) ||
+		           (delta > 0 && new_delta > 0 && new_delta < delta)) {
 			delta = new_delta;
 			thumbnail = current_thumb;
 			g_debug ("Choosing a %u pixel wide thumbnail (delta: %i) for entry %s",
-				 gdata_media_thumbnail_get_width (current_thumb), new_delta, id);
+			         gdata_media_thumbnail_get_width (current_thumb), new_delta, id);
 		}
 	}
 
@@ -779,52 +799,84 @@ query_progress_cb (GDataEntry *entry, guint entry_key, guint entry_count, QueryD
 
 		t_data = g_slice_new (ThumbnailData);
 		t_data->plugin = g_object_ref (self);
-		t_data->iter = iter;
+		t_data->path = gtk_tree_model_get_path (GTK_TREE_MODEL (self->list_store[data->tree_view]), &iter);
 		t_data->tree_view = data->tree_view;
 
+		/* We can use the same cancellable for reading the file and making a pixbuf out of it, as they're consecutive operations */
+		/* This will be cancelled if the main query is cancelled, in query_finished_cb() */
+		data->thumbnail_cancellable = g_cancellable_new ();
+		t_data->cancellable = g_object_ref (data->thumbnail_cancellable);
+
 		g_debug ("Starting thumbnail download for entry %s", id);
 		thumbnail_file = g_file_new_for_uri (gdata_media_thumbnail_get_uri (thumbnail));
-		g_file_read_async (thumbnail_file, G_PRIORITY_DEFAULT, self->cancellable[data->tree_view],
-				   (GAsyncReadyCallback) thumbnail_opened_cb, t_data);
+		g_file_read_async (thumbnail_file, G_PRIORITY_DEFAULT, data->thumbnail_cancellable,
+		                   (GAsyncReadyCallback) thumbnail_opened_cb, t_data);
 		g_object_unref (thumbnail_file);
 	}
 }
 
+/* Called when self->cancellable[tree_view] is destroyed (for either tree view) */
+static void
+cancellable_notify_cb (TotemYouTubePlugin *self, GCancellable *old_cancellable)
+{
+	guint i;
+
+	/* Disable the "Cancel" button, if it applies to the current tree view */
+	if (self->cancellable[self->current_tree_view] == old_cancellable)
+		gtk_widget_set_sensitive (self->cancel_button, FALSE);
+
+	/* NULLify the cancellable */
+	for (i = 0; i < NUM_TREE_VIEWS; i++) {
+		if (self->cancellable[i] == old_cancellable)
+			self->cancellable[i] = NULL;
+	}
+}
+
+static void
+set_current_operation (TotemYouTubePlugin *self, guint tree_view, GCancellable *cancellable)
+{
+	/* Cancel previous searches on this tree view */
+	if (self->cancellable[tree_view] != NULL)
+		g_cancellable_cancel (self->cancellable[tree_view]);
+
+	/* Make this the current cancellable action for the given tab */
+	g_object_weak_ref (G_OBJECT (cancellable), (GWeakNotify) cancellable_notify_cb, self);
+	self->cancellable[tree_view] = cancellable;
+
+	/* Enable the "Cancel" button if it applies to the current tree view */
+	if (self->current_tree_view == tree_view)
+		gtk_widget_set_sensitive (self->cancel_button, TRUE);
+}
+
 static void
 execute_query (TotemYouTubePlugin *self, guint tree_view, gboolean clear_tree_view)
 {
 	QueryData *data;
 
-	/* Cancel previous searches on this tree view */
-	if (self->cancellable[tree_view] != NULL) {
-		g_cancellable_cancel (self->cancellable[tree_view]);
-		g_object_unref (self->cancellable[tree_view]);
-	}
+	/* Set up the query */
+	data = g_slice_new (QueryData);
+	data->plugin = g_object_ref (self);
+	data->tree_view = tree_view;
+	data->query_cancellable = g_cancellable_new ();
+	data->t_param_cancellable = NULL;
+	data->thumbnail_cancellable = NULL;
+
+	/* Make this the current cancellable action for the given tab */
+	set_current_operation (self, tree_view, data->query_cancellable);
 
 	/* Clear the tree views */
 	if (clear_tree_view == TRUE)
 		gtk_list_store_clear (self->list_store[tree_view]);
 
-	/* Do the query */
-	self->cancellable[tree_view] = g_cancellable_new ();
-
-	data = g_slice_new (QueryData);
-	data->plugin = g_object_ref (self);
-	data->tree_view = tree_view;
-
 	if (tree_view == SEARCH_TREE_VIEW) {
-		gdata_youtube_service_query_videos_async (self->service, self->query[tree_view], self->cancellable[tree_view],
-							  (GDataQueryProgressCallback) query_progress_cb, data,
-							  (GAsyncReadyCallback) query_finished_cb, data);
+		gdata_youtube_service_query_videos_async (self->service, self->query[tree_view], data->query_cancellable,
+		                                          (GDataQueryProgressCallback) query_progress_cb, data,
+		                                          (GAsyncReadyCallback) query_finished_cb, data);
 	} else {
-		gdata_youtube_service_query_related_async (self->service, self->playing_video, self->query[tree_view], self->cancellable[tree_view],
-							   (GDataQueryProgressCallback) query_progress_cb, data,
-							   (GAsyncReadyCallback) query_finished_cb, data);
+		gdata_youtube_service_query_related_async (self->service, self->playing_video, self->query[tree_view], data->query_cancellable,
+		                                           (GDataQueryProgressCallback) query_progress_cb, data,
+		                                           (GAsyncReadyCallback) query_finished_cb, data);
 	}
-
-	/* Enable the "Cancel" button if it applies to the current tree view */
-	if (self->current_tree_view == tree_view)
-		gtk_widget_set_sensitive (self->cancel_button, TRUE);
 }
 
 void
@@ -879,7 +931,10 @@ search_button_clicked_cb (GtkButton *button, TotemYouTubePlugin *self)
 void
 cancel_button_clicked_cb (GtkButton *button, TotemYouTubePlugin *self)
 {
-	g_assert (self->cancellable[self->current_tree_view] != NULL);
+	/* It's possible for the operation to finish (and consequently the cancellable to disappear) while the GtkButton is deciding whether the
+	 * user is actually pressing it (in its timeout). */
+	if (self->cancellable[self->current_tree_view] == NULL)
+		return;
 
 	g_debug ("Cancelling search");
 	g_cancellable_cancel (self->cancellable[self->current_tree_view]);
@@ -935,7 +990,7 @@ open_in_web_browser_activate_cb (GtkAction *action, TotemYouTubePlugin *self)
 	for (path = paths; path != NULL; path = path->next) {
 		GtkTreeIter iter;
 		GDataYouTubeVideo *video;
-		GDataLink *link;
+		GDataLink *page_link;
 		GError *error = NULL;
 
 		if (gtk_tree_model_get_iter (model, &iter, (GtkTreePath*) (path->data)) == FALSE)
@@ -943,11 +998,12 @@ open_in_web_browser_activate_cb (GtkAction *action, TotemYouTubePlugin *self)
 
 		/* Get the HTML page for the video; its <link rel="alternate" ... /> */
 		gtk_tree_model_get (model, &iter, 3, &video, -1);
-		link = gdata_entry_look_up_link (GDATA_ENTRY (video), GDATA_LINK_ALTERNATE);
+		page_link = gdata_entry_look_up_link (GDATA_ENTRY (video), GDATA_LINK_ALTERNATE);
 		g_object_unref (video);
 
 		/* Display the page */
-		if (gtk_show_uri (gtk_widget_get_screen (GTK_WIDGET (self->bvw)), gdata_link_get_uri (link), GDK_CURRENT_TIME, &error) == FALSE) {
+		if (gtk_show_uri (gtk_widget_get_screen (GTK_WIDGET (self->bvw)), gdata_link_get_uri (page_link),
+		                  GDK_CURRENT_TIME, &error) == FALSE) {
 			GtkWindow *window = totem_get_main_window (self->totem);
 			totem_interface_error (_("Error Opening Video in Web Browser"), error->message, window);
 			g_object_unref (window);
@@ -1011,3 +1067,4 @@ starting_video_cb (TotemVideoList *video_list, GtkTreePath *path, TotemYouTubePl
 
 	return TRUE;
 }
+
