@@ -87,12 +87,18 @@ GtkWidget *totem_playlist_new      (void);
  * @display_name is if you have a preferred display string for the mrl,
  * NULL otherwise
  */
-gboolean totem_playlist_add_mrl  (TotemPlaylist *playlist,
-				  const char *mrl,
-				  const char *display_name);
-gboolean totem_playlist_add_mrl_with_cursor (TotemPlaylist *playlist,
-					     const char *mrl,
-					     const char *display_name);
+void totem_playlist_add_mrl (TotemPlaylist *playlist,
+                             const char *mrl,
+                             const char *display_name,
+                             gboolean cursor,
+                             GCancellable *cancellable,
+                             GAsyncReadyCallback callback,
+                             gpointer user_data);
+gboolean totem_playlist_add_mrl_finish (TotemPlaylist *playlist,
+                                        GAsyncResult *result);
+gboolean totem_playlist_add_mrl_sync (TotemPlaylist *playlist,
+                                      const char *mrl,
+                                      const char *display_name);
 
 void totem_playlist_save_current_playlist (TotemPlaylist *playlist,
 					   const char *output);
@@ -111,7 +117,7 @@ char      *totem_playlist_get_current_mrl (TotemPlaylist *playlist,
 char      *totem_playlist_get_current_title (TotemPlaylist *playlist,
 					     gboolean *custom);
 char      *totem_playlist_get_title (TotemPlaylist *playlist,
-				     guint index);
+				     guint title_index);
 
 gboolean   totem_playlist_set_title (TotemPlaylist *playlist,
 				     const char *title,
@@ -142,7 +148,7 @@ void       totem_playlist_set_at_end (TotemPlaylist *playlist);
 
 int        totem_playlist_get_current (TotemPlaylist *playlist);
 int        totem_playlist_get_last (TotemPlaylist *playlist);
-void       totem_playlist_set_current (TotemPlaylist *playlist, guint index);
+void       totem_playlist_set_current (TotemPlaylist *playlist, guint current_index);
 
 void       totem_playlist_foreach (TotemPlaylist *playlist,
 				   TotemPlaylistForeachFunc callback,
